"use strict";
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var errors_1 = require("./errors");
exports.ConfigNotFound = errors_1.ConfigNotFound;
exports.ModuleNameNotProvided = errors_1.ModuleNameNotProvided;
var utilities_1 = require("./utilities");
/** A configuration file loader. */
var Disposition = /** @class */ (function () {
    /**
     * Instantiate a new instance of Disposition.
     *
     * @param moduleName The name that will be used for the
     *                   configuration file.
     * @param options The options object for configuring Dispostion's
     *                behaviour.
     */
    function Disposition(moduleName, options) {
        this.moduleName = moduleName;
        /**
         * An object containing configuration options for Dispostion itself.
         */
        this.options = {
            project: utilities_1.resolveSelf('tsconfig.json'),
            checkEmpty: true,
        };
        // While TypeScript users will be protected against this, users
        // of plain JavaScript will not.
        if (this.moduleName === undefined) {
            throw new errors_1.ModuleNameNotProvided();
        }
        if (options !== undefined) {
            this.options = lodash_1.merge(this.options, options);
        }
        this.filenames = [
            this.moduleName + ".config.js",
            this.moduleName + ".config.ts",
            this.moduleName + ".config.tsx",
        ];
    }
    /**
     * Search for and load the configuation file.
     *
     * The filenames that are searched for are based off
     * the following template: ${moduleName.config.(js|ts|tsx)
     *
     * @param directory The directory to start searching from. Defaults
     *                  to the current working directory.
     */
    Disposition.prototype.load = function (directory) {
        if (directory === void 0) { directory = process.cwd(); }
        var configPath = utilities_1.find(directory, this.filenames);
        if (configPath === undefined) {
            throw new errors_1.ConfigNotFound(this.filenames);
        }
        return this.require(configPath);
    };
    /**
     * Search for and load the specified configuration file.
     *
     * @param path The path to the configuration file.
     */
    Disposition.prototype.loadFile = function (path) {
        return this.require(path);
    };
    /**
     * Require the configuration file and return the required object.
     *
     * @param path The path to the configuration file.
     */
    Disposition.prototype.require = function (path) {
        require('ts-node').register({
            project: this.options.project,
            // Ensure that the version of TypeScript shipped with Disposition
            // is used.
            compiler: require.resolve('typescript'),
            compilerOptions: {
                // Allow compiling and running files written in JavaScript.
                allowJs: true,
            },
            ignore: ['/node_modules/'],
        });
        var config = require(path);
        if (config.__esModule === true) {
            this.errorOnEmptyConfig(config.default);
            return config.default;
        }
        else {
            this.errorOnEmptyConfig(config);
            return config;
        }
    };
    /**
     * Check if the provided configuration file is empty, and
     * throw an error if it is.
     */
    Disposition.prototype.errorOnEmptyConfig = function (config) {
        if (this.options.checkEmpty && lodash_1.isEmpty(config)) {
            throw new errors_1.EmptyConfig();
        }
    };
    return Disposition;
}());
exports.default = Disposition;

//# sourceMappingURL=disposition.js.map
